/*
 * Traditional system functions for memory management.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "sys_memory.h"

#include <errno.h>
#include <malloc.h>


/* Obtain an aligned memory region of the given alignment and size, populating
   the given pointer. The alignment must express a whole power of two and
   express a multiple of sizeof(void *). */

int posix_memalign(void **memptr, size_t alignment, size_t size)
{
  void *addr;

  /* Require a power of two (a single bit set only) and a multiple of the void
     pointer size. */

  if (!alignment || ((alignment & (alignment - 1))) || (alignment % sizeof(void *)))
    return EINVAL;

  if (size)
  {
    addr = _memalign_r(_REENT, alignment, size);

    if (addr == NULL)
      return ENOMEM;

    *memptr = addr;
  }
  else
    *memptr = NULL;

  return 0;
}
